#include "handmade.h"

#define PI32 3.141592653589f

internal int32 roundReal32ToUInt32(real32 realNum) {
    return (uint32)(realNum + 0.5f);
}

internal int32 roundReal32ToInt32(real32 realNum) {
    return (int32)(realNum + 0.5f);
}

internal void drawRectangle(GameOffscreenBuffer *buffer, real32 realMinX, real32 realMinY, real32 realMaxX, real32 realMaxY, real32 R, real32 G, real32 B) {
    int32 minX = roundReal32ToInt32(realMinX);
    int32 minY = roundReal32ToInt32(realMinY);
    int32 maxX = roundReal32ToInt32(realMaxX);
    int32 maxY = roundReal32ToInt32(realMaxY);

    uint32 color = (roundReal32ToUInt32(R*255.0f) << 16) |
                   (roundReal32ToUInt32(G*255.0f) <<  8) |
                   (roundReal32ToUInt32(B*255.0f) <<  0);

    if (minX < 0) {
        minX = 0;
    }

    if (minY < 0) {
        minY = 0;
    }

    if (maxX > buffer->width) {
        maxX = buffer->width;
    }

    if (maxY > buffer->height) {
        maxY = buffer->height;
    }

    uint8 *endOfBuffer = (uint8 *)buffer->memory + buffer->pitch*buffer->height;
    uint8 *row = ((uint8 *)buffer->memory + minX*buffer->bytesPerPixel + minY*buffer->pitch);
    for (int y = minY; y < maxY; y++) {
        uint32 *pixel = (uint32 *)row;
        for (int x = minX; x < maxX; x++) {
            *pixel++ = color;
        }
        row += buffer->pitch;
    }
}

internal void outputSound(GameSoundOutputBuffer *soundBuffer, GameState *state) {
    int16 toneVolume = 3000;
    int16 *sampleOut = soundBuffer->samples;
    for (int sampleIndex = 0; sampleIndex < soundBuffer->sampleCount; sampleIndex++) {
        int16 sampleValue = 0;
        *sampleOut++ = sampleValue;
        *sampleOut++ = sampleValue;
    }
}

extern "C" GAME_UPDATE_AND_RENDER(gameUpdateAndRender) {
    Assert(sizeof(GameState) <= memory->permanentStorageSize);

    GameState *state = (GameState*)memory->permanentStorage;

    if (!memory->isInitialised) {
        memory->isInitialised = true;
        state->playerX = 20.0f;
        state->playerY = 20.0f;
    }

    for (int controllerIndex = 0; controllerIndex < ArrayCount(input->controllers); controllerIndex++) {
        GameControllerInput *controller = &input->controllers[controllerIndex];

        real32 dPlayerX = 0.0f;
        real32 dPlayerY = 0.0f;

        if (controller->stickUp.endedDown) {
            dPlayerY = -128.0f;
        }
        if (controller->stickDown.endedDown) {
            dPlayerY = 128.0f;
        }
        if (controller->stickLeft.endedDown) {
            dPlayerX = -128.0f;
        }
        if (controller->stickRight.endedDown) {
            dPlayerX = 128.0f;
        }

        state->playerX += dPlayerX * input->dtForFrame;
        state->playerY += dPlayerY * input->dtForFrame;
    }

    const int TILEMAP_WIDTH = 16;
    const int TILEMAP_HEIGHT = 9;
    real32 upperLeftX = 0;
    real32 upperLeftY = 0;
    real32 tileWidth = 60;
    real32 tileHeight = 60;

    uint32 tileMap[TILEMAP_HEIGHT][TILEMAP_WIDTH] = {
        {1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1},
        {1, 0, 0, 0,  0, 1, 0, 0,  0, 0, 0, 0,  0, 1, 0, 1},
        {1, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 1, 1},
        {1, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 1},
        {1, 0, 0, 0,  0, 1, 0, 0,  0, 0, 0, 0,  0, 0, 1, 1},
        {1, 0, 0, 0,  0, 1, 0, 0,  0, 0, 0, 0,  0, 1, 0, 1},
        {1, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,  1, 0, 0, 1},
        {1, 0, 0, 0,  0, 0, 0, 0,  0, 0, 0, 0,  0, 1, 0, 1},
        {1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1,  1, 1, 1, 1},
    };

    // clearscreen
    drawRectangle(videoBuf, 0.0f, 0.0f, (real32)videoBuf->width, (real32)videoBuf->height, 1.0f, 0.0f, 1.0f);

    // tilemap
    for (int row = 0; row < TILEMAP_HEIGHT; row++) {
        for (int col = 0; col < TILEMAP_WIDTH; col++) {
            uint32 tileID = tileMap[row][col];
            real32 fill = tileID == 1 ? 1.0f : 0.5f;
            real32 minX = upperLeftX + ((real32)col)*tileWidth;
            real32 minY = upperLeftY + ((real32)row)*tileHeight;
            real32 maxX = minX + tileWidth;
            real32 maxY = minY + tileHeight;
            drawRectangle(videoBuf, minX, minY, maxX, maxY, fill, fill, fill);
        }
    }

    // player
    real32 playerR = 1.0f;
    real32 playerG = 0.5f;
    real32 playerB = 0.0f;
    real32 playerWidth = 25.0f;
    real32 playerHeight = 40.0f;
    real32 playerLeft = state->playerX - 0.5f * playerWidth;
    real32 playerTop = state->playerY - 0.5f * playerHeight;

    drawRectangle(videoBuf, playerLeft, playerTop, state->playerX + playerWidth, state->playerY + playerHeight, playerR, playerG, playerB);
}

extern "C" GAME_GET_SOUND_SAMPLES(gameGetSoundSamples) {
    outputSound(soundBuf, (GameState*)memory->permanentStorage);
}
